;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  Common Lisp ݊pbP[W
;;;
;;;  
;;; All Right Reserved, 2001 (C) CopyRight by GOMI Hiroshi
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 7-Feb-2001: starts
;;;  Xg: nconc, append, first, second, third, cadr, cddr, nth, nthcdr, 
;;;          reverse, remove, member, mapcar, get, put, replace, push, pop
;;;  :     zerop, 1+, 1-, plusp, minusp, inc, dec
;;;  q:   null, not, integerp, floatp, numberp, stringp, symbolp, threadp,
;;;          errorp,
;;;          equal
;;;  F  break, let

;;;
;;; Xg %list
;;;               

;;; nconc --- same as Common Lisp
(defun nconc (%list1 %list2)
   (if (null %list1) %list2 (nconc2 %list1 %list2)))
(defun nconc2 (%list1 %list2)
   (if (atom (cdr %list1))
       (set-cdr %list1 %list2)
       (nconc2 (cdr %list1) %list2))
   %list1))

;;; append --- same as Common Lisp
(defun append (%list1 %list2) (nconc (copy-list %list1) %list2))

;;; first, second, third, cadr, cddr --- same as Common Lisp
(defun first (list) (car list))

(defun second (list) (car (cdr list)))

(defun third (list) (car (cdr (cdr list))))

(defun cadr (list) (car (cdr list)))

(defun cddr (list) (cdr (cdr list)))

;;; nth --- same as Common Lisp
(defun nth (%index list)
   (if (zerop %index) 
       (car list)
       (nth (1- %index) (cdr list))))

(defun nthcdr (%index list)
   (if (zerop %index) 
       (cdr list)
       (nthcdr (1- %index) (cdr list))))

(defun reverse (list)
   (if (null list)
       nil
       (add-list (reverse (cdr list)) (car list))))

(defun remove (%item list)
   (if (null list) 
       nil
       (if (= %item (car list))
           (remove %item (cdr list))
           (cons (car list) (remove %item (cdr list)))))) 

(defun member (%item list)
   (if (null list)
       nil
       (if (= %item (car list))
           list
           (member %item (cdr list)))))           

(defun mapcar (%fun %list)
   (if (atom %list)
       %list
       (cons (apply %fun (list (car %list)))
             (mapcar %fun (cdr %list)))))

(defun put (%key %value %alist)
   (if (null alist%)
       (list (cons %key %value))
       (put-aux %key %value %alist)))
(defun put-aux (%key %value %alist)
   (if (= %key (car (car %alist)))
       (set-cdr (car %alist) %value)
       (if (null (cdr %alist))
           (set-cdr %alist (list (cons %key %value)))
           (put-aux %key %value (cdr %alist))))
   %alist)

(defun get (%key %alist)
   (if (null %alist)
       nil
       (if (= %key (car (car %alist)))
           (cdr (car %alist))
           (get %key (cdr %alist))))) 

;;;
;;; Replace
;;;   }NWJɕKv
(defun replace (%new %old %body)       
   (if (null %old)
       %body
       (if (symbolp %old)
           (replace-symbol %new %old %body)
           (replace (cdr %new) 
                    (cdr %old) 
                    (replace (car %new) (car %old) %body)))))
(defun replace-symbol (%new %old %body)
   (if (eq %body %old)
       %new
       (if (atom %body)
           %body
           (cons (replace-symbol %new %old (car %body))
                 (replace-symbol %new %old (cdr %body))))))

(defmacro push (%item %list)
   (setq %list (cons %item %list)))

(defmacro pop (%list)
   (setq %list (cdr %list)))

;;;
;;; Number
;;;       
(defun zerop (number) (= number 0))

(defun 1- (number) (- number 1))

(defun 1+ (number) (+ number 1))

(defun plusp (number) (> number 0))

(defun minusp (number) (< number 0))

(defmacro inc (number) (setq number (1+ number)))

(defmacro dec (number) (setq number (1- number)))

;;;
;;; q
;;;
(defun null (%obj) (eq (type-of %obj) 'warabi.nil))

(defun not (%obj) (null %obj))

(defun integerp (%obj) (eq (type-of %obj) 'warabi.integer))

(defun floatp (%obj) (eq (type-of %obj) 'warabi.float))

(defun numberp (%obj) (if (integerp %obj) t (floatp %obj)))

(defun stringp (%obj) (eq (type-of %obj) 'warabi.string))

(defun symbolp (%obj) (eq (type-of %obj) 'warabi.symbol))

(defun threadp (%obj) (eq (type-of %obj) 'warabi.lispthread))

(defun errorp (%obj) (eq (type-of %obj) 'warabi.error))

(defun equal (%list1 %list2)
   (if (atom %list1)
       (if (= %list1 %list2) t nil)
       (if (atom %list2) 
           nil
           (if (null %list1)
               (null %list2)
               (if (null %list2)
                   nil
                   (if (equal (car %list1) (car %list2))
                       (equal (cdr %list1) (cdr %list2))
                       nil))))))

;;;
;;; BREAK
;;;
(defun break ()
   (let (ok)
     (progn
       (setq ok 'ok)
       (loop 
          (write "Break>")
          (setq %input (read))
          (if (null %input) (return 'ok))
          (if (eq %input 'ok) (return 'ok))
          (catch nil 
             (print (eval %input)))))))

;;;
;;; LET
;;;   (let (x y z) (foo x y z)) ̌`̂݃T|[g
;;;   --> ((lambda (x y z) (foo x y z)) '(nil nil nil))
(defmacro let (%binding %form)
    ((lambda %binding %form)
     (generate-binding (quote %binding))))

(defun generate-binding (%binding)
    (mapcar '(lambda (%x) nil) %binding))
